import os
from launch import LaunchDescription
from launch_ros.actions import Node
from launch.actions import ExecuteProcess, DeclareLaunchArgument
from ament_index_python.packages import get_package_share_directory
from launch.substitutions import LaunchConfiguration

def generate_launch_description():

    package_name = 'simple_robot_2dof'

    rviz_config_file = os.path.join(
        get_package_share_directory(package_name),
        'rviz',
        'forward_kinematics_simulations.rviz'
    )

    velocity1_launch_arg = DeclareLaunchArgument('velocity1', default_value = '0.01') # constant: 0.015 acceleration: 0.005
    velocity1 = LaunchConfiguration('velocity1')

    velocity2_launch_arg = DeclareLaunchArgument('velocity2', default_value = '0.03')
    velocity2 = LaunchConfiguration('velocity2')

    velocity3_launch_arg = DeclareLaunchArgument('velocity3', default_value = '0.005')
    velocity3 = LaunchConfiguration('velocity3')

    slope_launch_arg = DeclareLaunchArgument('slope', default_value = '0.25') # [0 ; 0.5]
    slope = LaunchConfiguration('slope')

    velocity_constant_launch_arg = DeclareLaunchArgument('velocity_constant', default_value = '0.015')
    velocity_constant = LaunchConfiguration('velocity_constant')

    sequential_launch_arg = DeclareLaunchArgument('sequential', default_value = 'True') # do not change
    sequential = LaunchConfiguration('sequential')

    constant_speed_launch_arg = DeclareLaunchArgument('constant_speed', default_value = 'True')
    constant_speed = LaunchConfiguration('constant_speed')

    return LaunchDescription([
        velocity1_launch_arg,
        velocity2_launch_arg,
        slope_launch_arg,
        velocity3_launch_arg,
        velocity_constant_launch_arg,
        constant_speed_launch_arg,
        sequential_launch_arg,
        Node(
            package = package_name,
            executable = 'forward_kinematics_node_2dof_2',
            name =  'forward_kinematics_node',
            output = 'screen',
            parameters = []
        ),
        Node(
            package = package_name,
            executable = 'reference_node_2dof',
            name =  'reference_node',
            output = 'screen',
            parameters = [
                {'velocity1': velocity1},
                {'velocity2': velocity2},
                {'velocity3': velocity3},
                {'velocity_constant': velocity_constant},
                {'slope': slope},
                {'sequential': sequential},
                {'constant_speed': constant_speed}
            ]
        ),
        Node(
            package='rviz2',
            executable='rviz2',
            name='rviz2',
            output='screen',
            arguments=['-d', rviz_config_file]
        ),
        Node(
            package= 'iiwa_inverse_kinematics',
            executable='inverse_kinematics_node',
            name='inverse_kinematics',
            output='screen',
            parameters = []
        )
    ])